/**
 * Copyright (C) 2001-2006 France Telecom R&D
 */
package common;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.Properties;

import javax.jdo.JDOHelper;
import javax.jdo.PersistenceManagerFactory;

public abstract class MainHelper {

    public PersistenceManagerFactory pmf = null;

    public Logger logger;

    public MainHelper() {
    }
    public MainHelper init(String[] args) {
        Map jdoConfig = getJdoConfig(args);
        pmf = JDOHelper.getPersistenceManagerFactory(jdoConfig);
        logger = Monolog.initialize().getLogger(getClass().getName());
        logger.setIntLevel(BasicLevel.INFO);
        logger.log(BasicLevel.INFO, "PersistenceManagerFactory fetched: "
            + pmf.getConnectionURL());
        return this;
    }
    public abstract MainHelper run();
    
    public MainHelper end() {
        logger.log(BasicLevel.INFO, "Closing the PersistenceManagerFactory: "
                + pmf.getConnectionURL());
        pmf.close();
        return this;
    }
    
    /**
     * Loads the jdo configuration file from the classpath
     * @param args is the arguments of the java application. The first 
     * parameter could the name of the jdo configuration file to use 
     * @return
     */
    public Map getJdoConfig(String[] args) {
        String configFile;
        InputStream is;
        if (args != null && args.length > 0) {
            configFile = args[0];
            is = getClass().getClassLoader().getResourceAsStream(configFile);
            if (is == null) {
                throw new IllegalArgumentException(
                        "No JDO configuration file found from the classpath: " + configFile);
            }
        } else {
            configFile = "speedo-jdo.properties";
            is = getClass().getClassLoader().getResourceAsStream(configFile);
            if (is == null) {
                configFile = "speedo-jdo.properties";
                is = getClass().getClassLoader().getResourceAsStream(configFile);
                if (is == null) {
                    throw new IllegalArgumentException(
                            "Impossible to find speedo configuration file from the class path (search speedo-jdo.properties or speedo.properties)");
                }
            }
        }
        //File has been found from the classpath
        //now load it into a Properties
        Properties pmfProp = new Properties();
        try {
            pmfProp.load(is);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return pmfProp;
    }
}
