/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.speedo.ejb;

import javax.persistence.PersistenceException;

import org.objectweb.speedo.lib.Personality;
import org.objectweb.speedo.pm.ejb.api.EJBPOManagerFactoryItf;
import org.objectweb.speedo.pm.ejb.api.EJBPOManagerItf;

public final class EJBPersonality extends Personality {

	private static final long serialVersionUID = 9783432421432544L;

    public RuntimeException newRuntimeException() {
    	return new PersistenceException();
    }
    public RuntimeException newRuntimeException(String msg) {
    	return new PersistenceException(msg);
    }
    public RuntimeException newRuntimeException(String msg, Throwable[] nested) {
    	if (nested.length == 0) {
    		return new PersistenceException(msg);
    	}
    	return new PersistenceException(msg, nested[0]);
    }
    public RuntimeException newRuntimeException(String msg, Throwable nested) {
    	return new PersistenceException(msg, nested);
    }
    public RuntimeException newRuntimeException(String msg, Object failed) {
    	return new PersistenceException(msg);
    }
    public RuntimeException newRuntimeException(String msg, Throwable[] nested, Object failed) {
    	if (nested.length == 0) {
    		return new PersistenceException(msg);
    	}
    	return new PersistenceException(msg, nested[0]);
    }
    public RuntimeException newRuntimeException(String msg, Throwable nested, Object failed) {
    	return new PersistenceException(msg, nested);
    }
    public RuntimeException newRuntimeException(Throwable nested) {
    	return new PersistenceException(nested);
    }
    public Class getRuntimeExceptionClass() {
    	return PersistenceException.class;
    }
    
    public RuntimeException newUserRuntimeException() {
    	return new PersistenceException();
    }
    public RuntimeException newUserRuntimeException(String msg) {
    	return new PersistenceException(msg);
    }
    public RuntimeException newUserRuntimeException(String msg, Throwable[] nested) {
    	if (nested.length == 0) {
    		return new PersistenceException(msg);
    	}
    	return new PersistenceException(msg, nested[0]);
    }
    public RuntimeException newUserRuntimeException(String msg, Throwable nested) {
    	return new PersistenceException(msg, nested);
    }
    public RuntimeException newUserRuntimeException(String msg, Object failed) {
    	return new PersistenceException(msg);
    }
    public RuntimeException newUserRuntimeException(String msg, Throwable[] nested, Object failed) {
    	if (nested.length == 0) {
    		return new PersistenceException(msg);
    	}
    	return new PersistenceException(msg, nested[0]);
    }
    public RuntimeException newUserRuntimeException(String msg, Throwable nested, Object failed) {
    	return new PersistenceException(msg, nested);
    }
    public RuntimeException newUserRuntimeException(Throwable nested) {
    	return new PersistenceException(nested);
    }
    public Class getUserRuntimeExceptionClass() {
    	return PersistenceException.class;
    }
    
    public RuntimeException newFatalRuntimeException() {
    	return new PersistenceException();
    }
    public RuntimeException newFatalRuntimeException(String msg) {
    	return new PersistenceException(msg);
    }
    public RuntimeException newFatalRuntimeException(String msg, Throwable[] nested) {
    	if (nested.length == 0) {
    		return new PersistenceException(msg);
    	}
    	return new PersistenceException(msg, nested[0]);
    }
    public RuntimeException newFatalRuntimeException(String msg, Throwable nested) {
    	return new PersistenceException(msg, nested);
    }
    public RuntimeException newFatalRuntimeException(String msg, Object failed) {
    	return new PersistenceException(msg);
    }
    public RuntimeException newFatalRuntimeException(String msg, Throwable[] nested, Object failed) {
    	if (nested.length == 0) {
    		return new PersistenceException(msg);
    	}
    	return new PersistenceException(msg, nested[0]);
    }
    public RuntimeException newFatalRuntimeException(String msg, Throwable nested, Object failed) {
    	return new PersistenceException(msg, nested);
    }
    public RuntimeException newFatalRuntimeException(Throwable nested) {
    	return new PersistenceException(nested);
    }
    public Class getFatalRuntimeExceptionClass() {
    	return PersistenceException.class;
    }
	public RuntimeException newFatalDataStoreRuntimeException() {
    	return new PersistenceException();
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg, Object failed) {
    	return new PersistenceException(msg);
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg, Throwable nested, Object failed) {
    	return new PersistenceException(msg, nested);
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg, Throwable nested) {
    	return new PersistenceException(msg, nested);
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg, Throwable[] nested, Object failed) {
    	if (nested.length == 0) {
    		return new PersistenceException(msg);
    	}
    	return new PersistenceException(msg, nested[0]);
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg, Throwable[] nested) {
    	if (nested.length == 0) {
    		return new PersistenceException(msg);
    	}
    	return new PersistenceException(msg, nested[0]);
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg) {
    	return new PersistenceException(msg);
	}
	public RuntimeException newFatalDataStoreRuntimeException(Throwable nested) {
    	return new PersistenceException(nested);
	}
	
	public RuntimeException newDataStoreRuntimeException(String msg, Throwable nested) {
		return new PersistenceException(msg, nested);
	}
	public RuntimeException newDataStoreRuntimeException(String msg) {
    	return new PersistenceException(msg);
	}
	public Class getDetachedFieldAccessRuntimeExceptionClass() {
    	return PersistenceException.class;
	}
	public Class getPOMClass() {
		return EJBPOManagerItf.class;
	}
	public Class getPOMFClass() {
		return EJBPOManagerFactoryItf.class;
	}
}
