/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass;

import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.speedo.mim.api.StateItf;
import org.objectweb.speedo.pm.api.POManagerItf;

import java.io.Serializable;

/**
 * Represents a element of a gen class. It matches to a reference element or
 * a primitive element.
 *
 * @author S.Chassande-Barrioz
 */
public interface GenClassElement extends PIndexedElem, Serializable {

    /**
     * @return the index of this gen class element, possibly wrapped in an object.
     */
    Object getIndex();

    /**
     * Assignes the index of the element of the gen class. It is a user object.
     * @param index to add, it cannot be null.
     */
    void setIndex (Object index);

    /**
     * @return the element of the gen class. It is a user object.
     */
    Object getElement();

    /**
     * Assignes the element of the gen class. It is a user object.
     * @param element to add, it cannot be null.
     */
    void setElement(Object element);

    /**
     * @param pm is the persistence manager which permits to resolve the PName
     * into a java reference.
     * @return the element of the gen class. The element is a reference
     * (PersistentObjectItf).
     */
    Object getElement(POManagerItf pm);

    /**
     * Assignes the jorm status of the PIndexedElement
     * @param s the new status
     */
    void setStatus(byte s);


    /**
     * Retrieves the Speedo accessor associated to this gen class element.
     */
    StateItf getSpeedoAccessor();

	/**
	 * In case of the element of the generic class is a persistent class. The
	 * implementation of this method should replace the direct reference to
	 * the persistent object by its identifier (PName). The aim of the 
	 * unswizzling feature is to permit to the cache to garbage unused instance.
	 * This unswizzling is done at commit time usualy.
	 */
    void unSwizzle();
	
	/**
	 * When thin lock is enabled (locking managed at genclass element level 
	 * instead of at genclass level), this method store the current status of
	 * the genclass element into another variable. This status is in fact the
	 * status of the genclass element during the working set. If the element
	 * is modified, the change(delta) has to be report on the reference state
	 * (cache) at commit time. 
	 * @return a byte indicating the retained status 
	 */
    byte retainStatusForMerge();
	
	/**
	 * Forget the retained status.
	 * @see #retainStatusForMerge()
	 */
    void cleanStatusForMerge();
	
	/**
	 * @return the retained status
	 * @see #retainStatusForMerge()
	 */
    byte getStatusForMerge();
	
	/**
	 * @return a clone of the current gen class element.
	 */
    GenClassElement cloneGCE();
}
