/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.genclass.api;

/**
 * This interface is a listener of generic class events. Two events are
 * listened: the adding or the removing of an  element into/from the genclass.
 */
public interface SpeedoGenClassListener {

    /**
     * Manage the coherence corresponding to the adding of an element into a
     * generic class.
     * @param elem is the element removed from the genclass.
     * @param gcid is the generic class identifier. A persistent class can have
     * several multivalued reference (generic class). Then this integer is the
     * identifier of the generic class in a persistent class.
     */
    void speedoElementAdded (Object elem, int gcid);

    /**
      * Manage the coherence corresponding to the adding of an element into a
      * generic class.
      * @param elem is the element added into the genclass.
      * @param gcid is the generic class identifier. A persistent class can have
      * several multivalued reference (generic class). Then this integer is the
      * identifier of the generic class in a persistent class.
      */
    void speedoElementRemoved (Object elem, int gcid);
}
