/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass.collection;

import org.objectweb.speedo.genclass.api.SpeedoGenClassPO;
import org.objectweb.speedo.mim.api.StateItf;

import java.util.Collection;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

/**
 *
 * @author S.Chassande-Barrioz
 */
public abstract class ArrayListImpl extends ArrayList
        implements SpeedoGenClassPO {

    public ArrayListImpl() {
        super();
		accessor = (ListAccessor) speedoCreateState();
    }

	ListAccessor accessor;

	public StateItf speedoGetReferenceState() {
		return accessor;
	}
	public void speedoSetReferenceState(StateItf refAcc) {
		accessor = (ListAccessor) refAcc;
	}

    // IMPLEMENTS THE HashSet CLASS //
    //------------------------------//

    public boolean add(Object o) {
        boolean result;
        if (!speedoIsActive()) {
            result = accessor.add(o);
        } else {
            ListAccessor ca = (ListAccessor) speedoGetHome().writeIntention(this, null);
            result = ca.add(o);
        }
        return result;
    }

    public boolean addAll(Collection c) {
        boolean result;
        if (!speedoIsActive()) {
            result = accessor.addAll(c);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().writeIntention(this, null);
            result = ca.addAll(c);
        }
        return result;
    }

    public void clear() {
        if (!speedoIsActive()) {
            accessor.clear();
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().writeIntention(this, null);
            ca.clear();
        }
    }

    public boolean contains(Object o) {
        if (!speedoIsActive()) {
            return accessor.contains(o);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.contains(o);
        }
    }

    public boolean containsAll(Collection c) {
        if (!speedoIsActive()) {
            return accessor.containsAll(c);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.containsAll(c);
        }
    }

    public boolean equals(Object o) {
        if (!speedoIsActive()) {
            return accessor.equals(o);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.equals(o);
        }
    }

    public boolean isEmpty() {
        if (!speedoIsActive()) {
            return accessor.isEmpty();
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.isEmpty();
        }
    }

    public Iterator iterator() {
        if (!speedoIsActive()) {
            return accessor.iterator();
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.iterator();
        }
    }

    public boolean remove(Object o) {
        boolean result;
        if (!speedoIsActive()) {
            result = accessor.remove(o);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().writeIntention(this, null);
            result = ca.remove(o);
        }
        return result;
    }

    public boolean removeAll(Collection c) {
        boolean result;
        if (!speedoIsActive()) {
            result = accessor.removeAll(c);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().writeIntention(this, null);
            result = ca.removeAll(c);
        }
        return result;
    }

    public boolean retainAll(Collection c) {
        throw new UnsupportedOperationException();
    }

    public int size() {
        if (!speedoIsActive()) {
            return accessor.size();
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.size();
        }
    }

    public Object[] toArray() {
        if (!speedoIsActive()) {
            return accessor.toArray();
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.toArray();
        }
    }

    public Object[] toArray(Object[] a) {
        if (!speedoIsActive()) {
            return accessor.toArray(a);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.toArray(a);
        }
    }

    public void add(int index, Object element) {
        if (!speedoIsActive()) {
            accessor.add(index, element);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            ca.add(index, element);
        }
    }
    public boolean addAll(int index, Collection c) {
        if (!speedoIsActive()) {
            return accessor.addAll(index, c);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.addAll(index, c);
        }
    }
    public Object get(int index) {
        if (!speedoIsActive()) {
            return accessor.get(index);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.get(index);
        }
    }
    public int indexOf(Object elem) {
        if (!speedoIsActive()) {
            return accessor.indexOf(elem);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.indexOf(elem);
        }
    }
    public Object remove(int index) {
        if (!speedoIsActive()) {
            return accessor.remove(index);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.remove(index);
        }
    }
    public Object set(int index, Object element) {
        if (!speedoIsActive()) {
            return accessor.set(index, element);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.set(index, element);
        }
    }
    public int lastIndexOf(Object o) {
        if (!speedoIsActive()) {
            return accessor.lastIndexOf(o);
        } else {
			ListAccessor ca = (ListAccessor) speedoGetHome().readIntention(this, null);
            return ca.lastIndexOf(o);
        }
    }
    public ListIterator listIterator() {
        if (!speedoIsActive()) {
            return ((List) accessor.collection).listIterator();
        } else {
			ListAccessor la = (ListAccessor) speedoGetHome().readIntention(this, null);
            return la.listIterator();
        }
    }

    public ListIterator listIterator(int i) {
        if (!speedoIsActive()) {
            return ((List) accessor.collection).listIterator(i);
        } else {
			ListAccessor la = (ListAccessor) speedoGetHome().readIntention(this, null);
            return la.listIterator(i);
        }
    }

    public List subList(int i, int i1) {
        if (!speedoIsActive()) {
            return ((List) accessor.collection).subList(i, i1);
        } else {
			ListAccessor la = (ListAccessor) speedoGetHome().readIntention(this, null);
            return la.subList(i, i1);
        }
    }
    public Object createGenClass() {
        return new ArrayList();
    }

    public StateItf speedoCreateState() {
        return new ListAccessor(this);
    }
}
