/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.generation.api;

import org.objectweb.speedo.api.SpeedoException;

/**
 * Defines a component of the Speedo generator.
 * 
 * @author S.Chassande-Barrioz
 */
public interface GeneratorComponent {

    /**
     * Assignes the parameters holder
     */
    void setSpeedoCompilerParameter(SpeedoCompilerParameter scp);

    /**
     * Retrieves the parameters holder
     */
    SpeedoCompilerParameter getSpeedoCompilerParameter();

    /**
     * Initializes the component
     * @return boolean value indicating if the process method can be called.
     * 
     * @throws SpeedoException if the component initialization fails
     */
    boolean init() throws SpeedoException;

    /**
     * Do the job of the component.
     * @throws SpeedoException if the component initialization fails
     */
    void process() throws SpeedoException;
    
    /**
     * @return a short string (less than 30 characters) representing the 
     * title of the component. This string must indicate job of the component.
     */
    String getTitle();
    
    /**
     * @return a short string (less than 50 characters) summarizing the job of
     * component (ex: how many files generated or compiled, ...)
     */
    String getSummary();

}
