/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.speedo.generation.ejb;

import org.objectweb.speedo.generation.AbstractEnhancer;
import org.objectweb.speedo.generation.api.GeneratorComponent;
import org.objectweb.speedo.generation.compiler.Compiler;
import org.objectweb.speedo.generation.enhancer.aware.PersistenceAwareEnhancer;
import org.objectweb.speedo.generation.enhancer.pc.PersistentClassEnhancer;
import org.objectweb.speedo.generation.generator.ejb.EJBGenerator;
import org.objectweb.speedo.generation.jorm.JormGenerator;
import org.objectweb.speedo.generation.jorm.JormMIGenerator;
import org.objectweb.speedo.generation.mivisitor.JMICleanerVisitor;
import org.objectweb.speedo.generation.mivisitor.ORMappingGenerator;
import org.objectweb.speedo.generation.mivisitor.ReverseFieldAdder;
import org.objectweb.speedo.generation.mivisitor.ejb.EJBSMIVisitor;
import org.objectweb.speedo.generation.parser.ejb.EJBAnnotationParser;
import org.objectweb.speedo.generation.parser.ejb.EJBPersistenceParser;
import org.objectweb.speedo.generation.recompiler.Recompiler;
import org.objectweb.speedo.generation.recompiler.UpToDateVisitor;
import org.objectweb.speedo.generation.serializer.MISerializer;
import org.objectweb.speedo.lib.Personality;

/**
 * This class is the enhancer for the EJB personality of Speedo.
 * 
 * @author S.Chassande-Barrioz, P. Dechamboux
 */
public class EJBEnhancer extends AbstractEnhancer {
    /**
     * Defines the different step of the Speedo EJB compiler/enhancer
     */
	private static final GeneratorComponent[] GC_GENCOMPS = new GeneratorComponent[] {
    	new EJBPersistenceParser(),
    	new UpToDateVisitor(Personality.EJB),
    	new Recompiler(Personality.EJB),
    	new EJBAnnotationParser(),
    	new EJBSMIVisitor(),
    	new ReverseFieldAdder(Personality.EJB),
    	new ORMappingGenerator(Personality.EJB),
    	new JormMIGenerator(Personality.EJB),
    	new JormGenerator(Personality.EJB),
    	new EJBGenerator(),
    	new MISerializer(Personality.EJB),
    	new PersistentClassEnhancer(Personality.EJB),
    	new PersistenceAwareEnhancer(Personality.EJB),
    	new JMICleanerVisitor(Personality.EJB),
    	new Compiler(Personality.EJB)		
	};
    
    protected GeneratorComponent[] getGeneratorComponents() {
        return GC_GENCOMPS;
    }

    public EJBEnhancer() {
    	super(Personality.EJB);
    }
}
