/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.enhancer.common;

import org.objectweb.asm.CodeAdapter;
import org.objectweb.asm.CodeVisitor;

/**
 * Modifies instructions that reference the old class in order to use the
 * new class name.
 *
 * @author S.Chassande-Barrioz
 */
public class CodeRenamer extends CodeAdapter {

    private final String oldClass;
    private final String classToWrite;
    
    /**
     * Creates a new {@link CodeRenamer}.
     * @param old is the old name of the class
     * @param neo is the new name of the class
     * @param cv
     *            the visitor to be used to generate the modified code
     */
    public CodeRenamer(final CodeVisitor cv, String old, String neo) {
        super(cv);
        this.oldClass = old;
        this.classToWrite = neo;
    }

    // IMPLEMENTATION OF THE CodeVisitor INTERFACE //
    // --------------------------------------------//

    public void visitTypeInsn(final int i, final String s) {
        final String type;
        if (oldClass.equals(s)) {
            type = classToWrite;
        } else {
            type = s;
        }
        cv.visitTypeInsn(i, type);
    }

    public void visitFieldInsn(final int opcode, final String owner,
            final String name, final String desc) {
        if (oldClass.equals(owner)) {
            cv.visitFieldInsn(opcode, classToWrite, name, desc);
        } else {
            cv.visitFieldInsn(opcode, owner, name, desc);
        }
    }

    public void visitMethodInsn(final int opcode, final String owner,
            final String name, final String desc) {
        if (oldClass.equals(owner)) {
            cv.visitMethodInsn(opcode, classToWrite, name, desc);
        } else {
            cv.visitMethodInsn(opcode, owner, name, desc);
        }
    }
}
