/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.generation.generator.api;

import org.apache.velocity.app.VelocityEngine;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.generation.api.GeneratorComponent;
import org.objectweb.speedo.metadata.SpeedoClass;

import java.util.Collection;

/**
 * This interface represents a component of a generator which generates a file
 * with the Velocity tool. The use of this interface permits to reuse the
 * VelocityEngine in order have better generation performance. Indeed the
 * VelocityEngine initialization is very cheap.
 *
 * @author S.Chassande-Barrioz
 */
public interface VelocityGenerator extends GeneratorComponent {

    /**
     * It assignes the VelocityEngine instance which must be used during the
     * veleocity generatioin.
     * @param ve is the VelocityEngine instance to use.
     */
    void setVelocityEngine(VelocityEngine ve);

    /**
     * @return the VelocityEngine used during the velocity generation.
     */
    VelocityEngine getVelocityEngine();

    /**
     * @return a collection of the file names which contains macros. This file
     * name list will be given to the VelocityEngine during its initialization.
     */
    Collection getExternalsTemplate();

    /**
     * It generates the file name corresponding to the Speedo meta object given
     * in parameter.
     * @param sClass is the speedo meta object which represents a persistent class
     * @param fileName is the generated file name
     * @throws SpeedoException
     */
    void generate(SpeedoClass sClass, String fileName) throws SpeedoException;
}
