/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.mivisitor;

import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.generation.lib.AbstractGeneratorComponent;
import org.objectweb.speedo.lib.Personality;

/**
 * Visit the speedo meta information in order to fill the meta information with
 * the part forgotten by the user. this class is the generic implementation. 
 * It must me extended for specializing the list of visitors.
 * 
 * @author S.Chassande-Barrioz
 */
public abstract class MetaInfoVisitors extends AbstractGeneratorComponent {
    public final static String LOGGER_NAME
            = SpeedoProperties.LOGGER_NAME + ".generation.compiler";
    /**
     * Is the first visitor starting the visit.
     */
    private AbstractMetaInfoVisitor firstVisitor;

    protected abstract AbstractMetaInfoVisitor[] getMIVisitors();
    
    public MetaInfoVisitors(Personality p) {
    	super(p);
    }
    
    public boolean init() throws SpeedoException {
	    if (scp.getXmldescriptor().isEmpty()) {
		    return false;
	    }
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
        AbstractMetaInfoVisitor[] modifiers = getMIVisitors();
        firstVisitor = new MetaInfoVisitorImpl(personality);
        firstVisitor.setSpeedoCompilerParameter(scp);
        firstVisitor.init();
        firstVisitor.setNext(modifiers[0]);
        for(int i=1; i<modifiers.length; i++) {
			modifiers[i-1].setSpeedoCompilerParameter(scp);
			modifiers[i-1].init();
            modifiers[i-1].setNext(modifiers[i]);
        }
	    return true;
    }
    public String getTitle() {
        return "Completing the meta data...";
    }

    public void process() throws SpeedoException {
        firstVisitor.visitCompilerParameter(scp);
    }
}
