/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.jmx;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * This class is a wrapper permiting to use Monolog as logging system, in MX4J.
 *
 * @author S.Chassande-Barrioz
 */
public class MX4JLoggerMonolog extends mx4j.log.Logger {
    
    /**
     * The inner monolog Logger.
     */
    Logger logger;
    
    /**
     * Builds a MX4JLoggerMonolog instance. MX4J requires an empty constructor. 
     */
    public MX4JLoggerMonolog() {
    }
    
    /**
     * Builds a MX4JLoggerMonolog with a particular inner Logger.
     * @param lo the inner logger
     */
    public MX4JLoggerMonolog(Logger lo) {
        this.logger = lo;
    }
    
    /**
     * Assignes the name/topic of the logger. The inner logger is allocated at
     * this time.
     */
    protected void setCategory(String name) {
        super.setCategory(name);
        logger = Monolog.monologFactory.getLogger(name);
    }
    
    /**
     * Assignes the priority/level on the logger.
     */
    public void setPriority(int p) {
        super.setPriority(p);
        logger.setIntLevel(priority2Level(p));
    }
    
    /**
     * Converts a priority (MX4J) to a level (Monolog)
     */
    private final static int priority2Level(int p) {
        switch (p) {
        case mx4j.log.Logger.FATAL:
            return BasicLevel.FATAL;
        case mx4j.log.Logger.ERROR:
            return BasicLevel.ERROR;
        case mx4j.log.Logger.WARN:
            return BasicLevel.WARN;
        case mx4j.log.Logger.INFO:
            return BasicLevel.INFO;
        case mx4j.log.Logger.DEBUG:
        default:
            return BasicLevel.DEBUG;
        }
    }
    
    /**
     * Log the MX4J event into the inner monolog logger.
     */
    protected void log(int p, Object m, Throwable t) {
        if (t == null) {
            logger.log(priority2Level(p), m);
        } else {
            logger.log(priority2Level(p), m, t);
        }
    }
}
