/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.jmx.mbeans;

import java.util.Collection;

import org.objectweb.perseus.cache.api.CacheAttributeController;
import org.objectweb.perseus.cache.api.CacheException;
import org.objectweb.perseus.cache.api.CacheManager;
import org.objectweb.perseus.cache.api.UnbindManager;
import org.objectweb.speedo.query.api.QueryManagerAttribute;

/**
 * 
 *
 * @author S.Chassande-Barrioz
 */
public class Query implements QueryMBean {

    private QueryManagerAttribute qma;
    private CacheAttributeController ca;
    private UnbindManager um;
    private CacheManager cm;

    public Query(CacheAttributeController ca,
            UnbindManager um,
            CacheManager cm,
           QueryManagerAttribute qma) {
        this.ca = ca;
        this.um = um;
        this.cm = cm;
        this.qma = qma;
    }

    public void evictAllCompiledQueriesFromCache() {
        try {
            um.unbindUnfixed(false);
        } catch (CacheException e) {
        }
    }
    
    public String getCompiledQueryCache_AutoCleanSize() {
        return ca.getAutoCleanSize();
    }
    public String getCompiledQueryCache_AutoCleanThreshold() {
        return ca.getAutoCleanThreshold();
    }
    public Collection getCompiledQueryCache_ObjectIdentifiers() {
        return ca.getCurrentEntryIdentifiers();
    }
    public int getCompiledQueryCache_CurrentSize() {
        return ca.getCurrentSize();
    }
    public int getCompiledQueryCache_MaxObjects() {
        return ca.getMaxObjects();
    }
    public void setCompiledQueryCache_AutoCleanSize(String size) {
        ca.setAutoCleanSize(size);
    }
    public void setCompiledQueryCache_AutoCleanThreshold(String size) {
        ca.setAutoCleanThreshold(size);
    }
    public void setCompiledQueryCache_MaxObjects(int size) throws IllegalArgumentException,
            CacheException {
    	ca.setMaxObjects(size);
    }

    // IMPLEMENTS THE QueryManagerAttribute INTERFACE //
    //------------------------------------------------//

    public boolean getPrefetchActivatedOnExtent() {
        return qma.getPrefetchActivatedOnExtent();
    }
    public boolean getPrefetchActivatedOnQuery() {
        return qma.getPrefetchActivatedOnQuery();
    }
    public void setPrefetchActivatedOnExtent(boolean prefetch) {
        qma.setPrefetchActivatedOnExtent(prefetch);
    }
    public void setPrefetchActivatedOnQuery(boolean prefetch) {
        qma.setPrefetchActivatedOnQuery(prefetch);
    }
}
