/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.lib;

import java.util.Locale;
import java.util.StringTokenizer;

import org.objectweb.speedo.api.UserFieldMapping;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class LocaleFieldMapping implements UserFieldMapping {

	/**
	 * Retrieves the java type corresponding to the type into the data support.
	 * @return a Class object (never null).
	 */
	public Class getStorageType() {
		return String.class;
	}

	/**
	 * Retrieves the java type corresponding to the type in memory.
	 * @return a Class object (never null).
	 */
	public Class getMemoryType() {
		return Locale.class;
	}

	/**
	 * Converts a value from the data support into a value in memory
	 * @param storagevalue is the value store in the support (can be null).
	 * @return the value in memory (can be null).
	 */
	public Object toMemory(Object storagevalue) {
		if (storagevalue == null) {
			return null;
		}
		
		if (storagevalue instanceof Locale) {
			return storagevalue;
		} else {
			StringTokenizer tokenizer = new StringTokenizer((String)storagevalue, "_");
			String[] names = new String[3];
			int max = tokenizer.countTokens();
			for (int i=0; i<3; i++) {				
				names[i] = "";
			}
			for (int i=0; i<max; i++) {
				String token = tokenizer.nextToken();
				names[i] = token;
			}
			
			Locale locale;			
			locale = new Locale(names[0], names[1], names[2]);			
			return locale;
		}
	}

	/**
	 * Converts a value from the memory into a value in data support 
	 * @param memoryvalue the value in memory (can be null).
	 * @return is the value store in the support (can be null).
	 */
	public Object toStorage(Object memoryvalue) {
		if (memoryvalue == null) {
			return null;
		}
		return ((Locale)memoryvalue).toString();		
	}
}
