/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.mapper.lib;

import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.mapper.api.ClassPropertyManager;
import org.objectweb.speedo.mim.api.HomeItf;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.util.Properties;


/**
 * 
 * 
 * @author S.Chassande-Barrioz
 */
public class CPMTransactionLockingLevel implements ClassPropertyManager, SpeedoProperties {

	public String getName() {
        return TRANSACTION_LOCKING_LEVEL;
	}

	public void applyDefault(HomeItf home, Properties props, Logger l) {
	}
	public void applyDefault(HomeItf gcHome, HomeItf refHome, Properties props, Logger l) {
	}
	public void applyProperty(String name, String value, 
			HomeItf home, Properties props, Logger logger) {
        applyProp(home, value, logger);
	}
    
	public void applyProperty(String name, String value, 
			HomeItf gcHome, HomeItf refHome, Properties props, Logger logger) {
        applyProp(gcHome, value, logger);
	}
    
    private void applyProp(HomeItf home, String value, Logger logger) {
        //Locking level= instance | field
        if (logger.isLoggable(BasicLevel.INFO)) {
            logger.log(BasicLevel.INFO, "Use the locking level '" + value 
                    + "' for the class '" + home.getPath());
        }
        if (TRANSACTION_LOCKING_LEVEL_FIELD.equalsIgnoreCase(value)) {
            home.setFieldLockingLevel(true);
        } else if (TRANSACTION_LOCKING_LEVEL_INSTANCE.equalsIgnoreCase(value)) {
            home.setFieldLockingLevel(false);
        } else {
            logger.log(BasicLevel.WARN, "Locking level '" + value 
                        + "' specified for the class '" + home.getPath() 
                        + "' is not managed.");
        }
    }
}
