/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.mapper.lib;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 * This class contains primitive to serialize and deserialize an Object into a
 * .jmi file.
 *
 * @author S.Chassande-Barrioz
 */
public class Object2JMIFileSerializer {

	public final static char[] ext = {'j', 'm', 'i'};

	public static String serialize(String output,
									 String descFileName,
									 Object o,
									 Logger logger) throws IOException {
		String fn = descFileName2FileName(descFileName);
		File f = new File(output, fn);
		logger.log(BasicLevel.DEBUG, "Store into the file: " + f.getAbsolutePath());
		f.getParentFile().mkdirs();
		ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(f));
		oos.writeObject(o);
		return f.getAbsolutePath();
	}

	public static Object deserialize(String descFileName, ClassLoader cl, Logger logger)
			throws Exception {
		String fn = descFileName2FileName(descFileName);
		InputStream is = cl.getResourceAsStream(fn);
		if (is == null) {
			Exception e = new IOException("File " + fn
					+ " is not availlable through the classloader (" + cl + ").");
			logger.log(BasicLevel.DEBUG, e.getMessage(), e);
			throw e;
		}
		logger.log(BasicLevel.DEBUG, "Load from the file: " + fn);
		ObjectInputStream ois = new ObjectInputStream(is);
		return ois.readObject();
	}

	public static String descFileName2FileName(String descFileName) {
		char[] cs = descFileName.toCharArray();
		int size = cs.length;
		cs[size - 1] = ext[2];
		cs[size - 2] = ext[1];
		cs[size - 3] = ext[0];
		for(int i=(size - 5); i>=0; i--) {
			if (cs[i] == '.') {
				cs[i] = File.separatorChar;
			}
		}
		return new String(cs);
	}
}
