/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.metadata;


/**
 * 
 * 
 * @author S.Chassande-Barrioz
 */
public class SpeedoColumn extends SpeedoElement {
	public String name;
	public String targetColumn;
	public String targetField;
	public String jdbcType;
	public String sqlType;
	public int length;
	public int scale;
	public boolean allowNull;
	public String defaultValue;
    public SpeedoTable table;
    public boolean insertable;
    public boolean updatable;
    
    public SpeedoColumn() {
		super();
    }
    public SpeedoColumn(String n) {
		this();
        this.name = n;
    }

    public SpeedoColumn(String n, SpeedoTable t) {
        this(n);
        this.table = t;
    }

    public Object clone() {
    	SpeedoColumn scol = new SpeedoColumn();
    	scol.name = name;
    	scol.targetColumn = targetColumn;
    	scol.targetField = targetField;
    	scol.jdbcType = jdbcType;
    	scol.sqlType = sqlType;
    	scol.length = length;
    	scol.scale = scale;
    	scol.allowNull = allowNull;
    	scol.defaultValue = defaultValue;
    	scol.table = table;
    	scol.insertable = insertable;
    	scol.updatable = updatable;
    	return scol;
    }

    public void merge(SpeedoColumn c) {
        if (name == null && c.name != null) {
            name = c.name;
        }
        if (targetColumn == null && c.targetColumn != null) {
            targetColumn = c.targetColumn;
        }
        if (targetField == null && c.targetField != null) {
            targetField = c.targetField;
        }
        if (jdbcType == null && c.jdbcType != null) {
            jdbcType = c.jdbcType;
        }
        if (sqlType == null && c.sqlType != null) {
            sqlType = c.sqlType;
        }
        if (length == -1 && c.length != -1) {
            length = c.length;
        }
        if (scale == -1 && c.scale != -1) {
            scale = c.scale;
        }
        allowNull = allowNull && c.allowNull;
        if (defaultValue == null && c.defaultValue != null) {
            defaultValue = c.defaultValue;
        }
        if (table == null && c.table != null) {
            table = c.table;
        }
    }
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("SpeedoColumn: name=").append(name);
        if (targetColumn != null) {
            sb.append(", targetColumn=").append(targetColumn);
        }
        if (targetField != null) {
            sb.append(", targetField=").append(targetField);
        }
        if (jdbcType != null) {
            sb.append(", jdbcType=").append(jdbcType);
        }
        if (sqlType != null) {
            sb.append(", sqlType=").append(sqlType);
        }
        if (defaultValue != null) {
            sb.append(", defaultValue=").append(defaultValue);
        }
        if (table != null) {
            sb.append(", table=").append(table.name);
        }
        if (length > 0) {
            sb.append(", length=").append(length);
        }
        if (scale > 0) {
            sb.append(", scale=").append(scale);
        }
        if (!allowNull) {
            sb.append(", allowNull=").append(allowNull);
        }
        if (!insertable) {
            sb.append(", insertable=").append(insertable);
        }
        if (!updatable) {
            sb.append(", updatable=").append(updatable);
        }
        return sb.toString();
    }
}
