/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.speedo.metadata;

import java.util.List;


/**
 * Defines a join between a main table and an external table.
 * 
 * @see org.objectweb.speedo.metadata.SpeedoTable
 * @see org.objectweb.speedo.metadata.SpeedoColumn
 * 
 * @author S.Chassande-Barrioz
 */
public class SpeedoJoin extends SpeedoElement {
	/**
	 * main table 
	 */
    public SpeedoTable mainTable;

    /**
     * external table
     */
    public SpeedoTable extTable;
	
	/**
	 * The list of SpeedoJoinColumn composing the join. This columns are in the 
     * external table.
     */
	public List columns;
	
	private final static byte ALL 			= (byte) 0xFF;
	private final static byte INDEXED 		= 1;
	private final static byte OUTER_JOIN 	= 2;
	private final static byte UNIQUE 		= 4;
	private byte conditions = 0;
	
	public final static byte ACTION_NONE 		= 1;
	public final static byte ACTION_NULL 		= 2;
	public final static byte ACTION_CASCADE 	= 3;
	public final static byte ACTION_RESTRICT 	= 4;
	public final static byte ACTION_DEFAULT 	= 5;
	
	/**
     * Defines the action to do on the external table when a delete action 
     * occurs in the main table. 
	 */
    public byte deleteAction;
    
    public SpeedoJoin() {
    	SpeedoDefaults.setDefaults(this);
    }

	public void setOuter(boolean v) {
		setCondition(v, OUTER_JOIN);
	}
	public boolean getOuter() {
		return getCondition(OUTER_JOIN);
	}
	
	public void setIndexed(boolean v) {
		setCondition(v, INDEXED);
	}
    /**
     * @return boolean value indicating if the join column are unique.
     */
	public boolean getIndexed() {
		return getCondition(INDEXED);
	}
	
	public void setUnique(boolean v) {
		setCondition(v, UNIQUE);
	}
	public boolean getUnique() {
		return getCondition(UNIQUE);
	}
	
	private void setCondition(boolean v, byte condition) {
		if (v) {
			conditions = (byte) (conditions | condition);
		} else {
			conditions = (byte) (conditions & (ALL - condition));
		}
	}
	private boolean getCondition(byte condition) {
		return (conditions & condition) == 1;
	}
	public String toString() {
	    StringBuffer sb = new StringBuffer();
	    sb.append("SpeedoJoin between '");
	    if (mainTable == null) {
	        sb.append("null");
	    } else {
	        sb.append(mainTable.name);
	    }
	    sb.append("' and '");
	    if (extTable == null) {
	        sb.append("null");
	    } else {
	        sb.append(extTable.name);
	    }
	    sb.append(" with this column").append(columns);
	    return sb.toString();
	}
}
