/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.speedo.metadata;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class SpeedoTable extends SpeedoElement {
	/**
	 * The name of the table.
	 */
    public String name;
    
    /**
     * The catalog of the table (null if none).
     */
    public String catalog;
    
    /**
     * The schema of the table (null if none).
     */
    public String schema;
    
    /**
     * If it is a secondary table, specify the join with the main table.
     */
    public SpeedoJoin join;
    
    /**
     * List of SpeedoIndexes
     */
    public List indexes = new ArrayList();
    
    /**
     * A list of unique constraints associated with the table.
     */
    public ArrayList uniqueConstraints = new ArrayList();
    
    public SpeedoTable() {
		SpeedoDefaults.setDefaults(this);
    }

    /**
     * Add a new unique constraint to this table. If it already exist, ignored.
     * @param cols	The list of columns specifying the constraint.
     * @return		false if it was already defined.
     */
    public boolean addUniqueConstraint(ArrayList cols) {
    	Iterator it = uniqueConstraints.iterator();
    	while (it.hasNext()) {
    		ArrayList al = (ArrayList) it.next();
    		Iterator it1 = cols.iterator();
    		Iterator it2 = al.iterator();
    		while (it1.hasNext() && it2.hasNext()) {
    			if (it1.next() != it2.next()) {
    				break;
    			}
    		}
    		if (it1.hasNext() || it2.hasNext()) {
    			continue;
    		}
    		return false;
    	}
    	uniqueConstraints.add(cols);
    	return true;
    }
    
    public boolean isMain() {
        return join == null;
    }
}
