/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.mim.api;

import org.objectweb.jorm.api.PBinding;
import org.objectweb.perseus.cache.replacement.api.ReplaceableCacheEntry;
import org.objectweb.speedo.mim.api.StateItf;
import org.objectweb.speedo.pm.api.POManagerItf;

/**
 * This interface what is a Peristent Object in Speedo. It is the jorm PBinding 
 * because it is unique. The Persistent object is an entry of the L2 cache.
 * A persistent object is linked to its home which is also the Mapping. A 
 * persistent object is linked to several state (StateIf) containing the 
 * persistent data.
 * 
 * @see org.objectweb.speedo.mim.api.HomeItf
 * @see org.objectweb.speedo.mim.api.StateItf
 * 
 * @author	S.Chassande-Barrioz
 */
public interface PersistentObjectItf
        extends PBinding, ReplaceableCacheEntry {

    /**
     * Tests whether the po infrastructure has already been set.
     * A po has a correct PClassMapping linked to a PBinder,
     * a PName obtained thanks to this PClassMapping.
     */
    boolean speedoIsActive();

    /**
     * Sets the new value of the po flags
     *
     * @see #speedoIsActive()
     * @param newvalue the new value of the po flag
     */
    void speedoIsActive(boolean newvalue);
    
    boolean speedoIsPersistent();
	
    HomeItf speedoGetHome();

    /**
     * It retrieves the StateItf instance used in the current context.
     * If the po is not active then the reference state is returned.
     * otherwise a $classNameFields is returned
     */
    StateItf speedoGetState();

    /**
     * @return the reference accessor of the PersistentObjectItf instance. It can be *
     * null.
     * The Reference accessor is used for non persistent instance, as cache 
     * value and for pessimistic policy. 
     */
    StateItf speedoGetReferenceState();

    /**
     * Assignes the reference accessor to the PersistentObjectItf
     * The Reference accessor is used for non persistent instance, as cache 
     * value and for pessimistic policy.
     *  
     * @param refAcc the new reference accessor
     */
    void speedoSetReferenceState(StateItf refAcc);

	/**
	 * @return a new StateItf instance.
	 */
    StateItf speedoCreateState();

	/**
	 * Copies persistent fields from an accessor to another.
	 * @param src is the accessor containing peristent fields to copy
	 * @param dest is the accessor where persistent fields have to be copied 
	 */
    void speedoCopyState(StateItf src, StateItf dest);

	/**
	 * Set the encoded PName of the detach copy to allow serialization.
	 * @param encodedPName : the encoded PName of the persistent object
	 */
	void speedoSetEncodedPName(Object encodedPName);
	
	/**
	 * @return the encoded PName of the detached copy
	 */
	Object speedoGetEncodedPName();
	
    /**
     * Retrieves the hints needed to build the PName of the persistent object
     * during the export action.
     */
    Object speedoGetPNameHints();

    /**
     * Adds an element in the gen class field whose number is given. This method
     * must call the speedoAdd method of the SpeedoGenClassCoherence interface,
     * or the Collection.add method (if the field contains a collection that
     * does implement the previous interface). If the field is null, a new
     * (speedo) collection must be created first.
     */
    void speedoAdd(Object elem, int gcFieldNumber);
    
    POManagerItf speedoGetPOManager(); 
}
