/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.pm.api;

import org.objectweb.speedo.lib.Personality;
import org.objectweb.speedo.query.api.QueryManager;
import org.objectweb.speedo.sequence.api.SequenceManager;

import java.util.Properties;

/**
 * It manages POManagerItf instances.
 *
 * @author S.Chassande-Barrioz
 */
public interface POManagerFactoryItf  {

	/**
	 * Finds in the associated POManagerSwitchItf a POManagerItf managed by the
	 * POManagerFactoryItf.
	 * @return a POManagerItf instance or a null value.
	 */
	POManagerItf lookup();

	/**
	 * It call pms.unbind(this) in order to unbind a POManagerItf managed by
	 * this POManagerFactoryItf, from the thread.
	 */
	void unbindPM();

	/**
	 * Ask to the PMS to bind a particular PM to the Thread.
	 * @param pm
	 */
	void bindPM2Thread(POManagerItf pm);

	/**
	 * Indicates when a PM has been closed.
	 * @param pr the closed PM.
	 */
	void poManagerClosed(POManagerItf pr);
	
	/**
	 * Get the sequence manager associated to the pmf.
	 */
	SequenceManager getSequenceManager();
	
	/**
	 * Set the sequence manager.
	 */
	void setSequenceManager(SequenceManager sequenceManager);
    
	/**
	 * Get the query manager.
	 */
	QueryManager getQueryManager();
	
	/**
	 * Set the query manager
	 */
	void setQueryManager(QueryManager queryManager);
	
    POManagerItf getPOManager();
    
    /**
     * Clean the POManagerFactory:
     * 	- it enables to forget all persistent classes previously loaded.
     */
    void clean();
    
    Personality getPersonality();
    
    Properties getProperties();
}
