/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2006 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 */

package org.objectweb.speedo.query.ejb;

import org.objectweb.speedo.query.api.QueryDefinition;

public class EJBQueryDefinitionImpl implements QueryDefinition {

    boolean withPrefetch;
    String query;
    long rangeFirst;
    long rangeLast;
    boolean fetchIdentifierOnly;
    short queryType;
    
    public EJBQueryDefinitionImpl() {
        defineWith(null);
    }
    public EJBQueryDefinitionImpl(EJBQueryDefinitionImpl qd) {
        defineWith(qd);
    }    
    public void defineWith(EJBQueryDefinitionImpl qd) {
        if (qd == null) {
            this.withPrefetch = true;
            this.query = null;
            this.rangeFirst = 0;
            this.rangeLast = Long.MAX_VALUE;
        } else {
            this.withPrefetch = qd.withPrefetch;
            this.query = qd.query;
            this.rangeFirst = qd.rangeFirst;
            this.rangeLast = qd.rangeLast;
        }
    }
    
    public boolean fetchIdentifierOnly() {
		return fetchIdentifierOnly;
	}
	public short getQueryType() {
		return queryType;
	}
	public boolean withPrefetch() {
        return withPrefetch;
    }

    public void withPrefetch(boolean withprefetch) {
        this.withPrefetch = withprefetch;
    }

    public long getIndexFirst() {
        return rangeFirst;
    }

    public long getIndexLast() {
        return rangeLast;
    }

    public String qdToString(boolean oneLine) {
        StringBuffer sb = new StringBuffer();
        String sep = "";
        final String sepNext = (oneLine ? ", " : "\n\t-");
        if (query != null) {
            sb.append(sep).append("query=\"").append(query).append("\"");
            sep = sepNext;
        }
        if (!withPrefetch) {
            sb.append(sep).append("withPrefetch=\"").append(withPrefetch).append("\"");
            sep = sepNext;
        }
        if (rangeFirst != 0) {
            sb.append(sep).append("range first=").append(rangeFirst);
            sep = sepNext;
        }
        if (rangeLast != 0) {
            sb.append(sep).append("range last=").append(rangeLast);
            sep = sepNext;
        }
        return sb.toString();
    }
}
