/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2006 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: A. Lefebvre
 */
package org.objectweb.speedo.query.ejb.parser;

import java.util.Stack;

import org.objectweb.speedo.query.ejb.parser.SimpleNode;
import org.objectweb.speedo.query.lib.SpeedoQueryHelper;

/**
 * @author Alexandre Lefebvre
 */
public class EJBQLAbstractVisitor extends SpeedoQueryHelper
	implements EJBQLVisitor {

    /**
     * ********************* VISITOR METHODS ***********************************
     */

    /**
     * Visit method to call from constructor.
     * Child node visitors get a <code>java.util.Stack</code> as data parameter.
     * @throws java.lang.Exception any nested exception thrown from other visit method
     */
    public Object visit(SimpleNode node) {
        return visit(node, new Stack());
    }

    /**
     * Generic visit method that traverses all child nodes
     */
    public Object visit(SimpleNode node, Object data) {
        return node.childrenAccept(this, data);
    }

    public Object visit(ASTEJBQL node, Object data) {
        return null;
    }

    public Object visit(ASTSelectStatement node, Object data) {
        return null;
    }

    public Object visit(ASTUpdateStatement node, Object data) {
        return null;
    }

    public Object visit(ASTDeleteStatement node, Object data) {
        return null;
    }

    public Object visit(ASTFromClause node, Object data) {
        return null;
    }

    public Object visit(ASTIdentificationVariableDeclaration node, Object data) {
        return null;
    }

    public Object visit(ASTRangeVariableDeclaration node, Object data) {
        return null;
    }

    public Object visit(ASTJoin node, Object data) {
        return null;
    }

    public Object visit(ASTFetchJoin node, Object data) {
        return null;
    }

    public Object visit(ASTJoinSpec node, Object data) {
        return null;
    }

    public Object visit(ASTCollectionMemberDeclaration node, Object data) {
        return null;
    }

    public Object visit(ASTPathExpression node, Object data) {
        return null;
    }

    public Object visit(ASTUpdateClause node, Object data) {
        return null;
    }

    public Object visit(ASTUpdateItem node, Object data) {
        return null;
    }

    public Object visit(ASTNewValue node, Object data) {
        return null;
    }

    public Object visit(ASTDeleteClause node, Object data) {
        return null;
    }

    public Object visit(ASTSelectClause node, Object data) {
        return null;
    }

    public Object visit(ASTSelectExpression node, Object data) {
        return null;
    }

    public Object visit(ASTConstructorExpression node, Object data) {
        return null;
    }

    public Object visit(ASTConstructorItem node, Object data) {
        return null;
    }

    public Object visit(ASTConstructorName node, Object data) {
        return null;
    }

    public Object visit(ASTAggregateSelectExpression node, Object data) {
        return null;
    }

    public Object visit(ASTWhereClause node, Object data) {
        return null;
    }

    public Object visit(ASTGroupByClause node, Object data) {
        return null;
    }

    public Object visit(ASTGroupByItem node, Object data) {
        return null;
    }

    public Object visit(ASTHavingClause node, Object data) {
        return null;
    }

    public Object visit(ASTOrderByClause node, Object data) {
        return null;
    }

    public Object visit(ASTOrderByItem node, Object data) {
        return null;
    }

    public Object visit(ASTSubQuery node, Object data) {
        return null;
    }

    public Object visit(ASTSubqueryFromClause node, Object data) {
        return null;
    }

    public Object visit(ASTSubselectIdentificationVariableDeclaration node, Object data) {
        return null;
    }

    public Object visit(ASTSimpleSelectClause node, Object data) {
        return null;
    }

    public Object visit(ASTSimpleSelectExpression node, Object data) {
        return null;
    }

    public Object visit(ASTConditionalExpression node, Object data) {
        return null;
    }

    public Object visit(ASTConditionalTerm node, Object data) {
        return null;
    }

    public Object visit(ASTConditionalFactor node, Object data) {
        return null;
    }

    public Object visit(ASTConditionalPrimary node, Object data) {
        return null;
    }

    public Object visit(ASTSimpleCondExpression node, Object data) {
        return null;
    }

    public Object visit(ASTBetweenExpression node, Object data) {
        return null;
    }

    public Object visit(ASTInExpression node, Object data) {
        return null;
    }

    public Object visit(ASTInItem node, Object data) {
        return null;
    }

    public Object visit(ASTLikeExpression node, Object data) {
        return null;
    }

    public Object visit(ASTNullComparisonExpression node, Object data) {
        return null;
    }

    public Object visit(ASTEmptyCollectionComparisonExpression node, Object data) {
        return null;
    }

    public Object visit(ASTCollectionMemberExpression node, Object data) {
        return null;
    }

    public Object visit(ASTExistsExpression node, Object data) {
        return null;
    }

    public Object visit(ASTAllOrAnyExpression node, Object data) {
        return null;
    }

    public Object visit(ASTComparisonExpression node, Object data) {
        return null;
    }

    public Object visit(ASTComparisonOperator node, Object data) {
        return null;
    }

    public Object visit(ASTArithmeticExpression node, Object data) {
        return null;
    }

    public Object visit(ASTSimpleArithmeticExpression node, Object data) {
        return null;
    }

    public Object visit(ASTArithmeticTerm node, Object data) {
        return null;
    }

    public Object visit(ASTArithmeticFactor node, Object data) {
        return null;
    }

    public Object visit(ASTArithmeticPrimary node, Object data) {
        return null;
    }

    public Object visit(ASTStringExpression node, Object data) {
        return null;
    }

    public Object visit(ASTStringPrimary node, Object data) {
        return null;
    }

    public Object visit(ASTDatetimeExpression node, Object data) {
        return null;
    }

    public Object visit(ASTDatetimePrimary node, Object data) {
        return null;
    }

    public Object visit(ASTBooleanExpression node, Object data) {
        return null;
    }

    public Object visit(ASTBooleanPrimary node, Object data) {
        return null;
    }

    public Object visit(ASTEntityExpression node, Object data) {
        return null;
    }

    public Object visit(ASTFunctionsReturningNumerics node, Object data) {
        return null;
    }

    public Object visit(ASTFunctionsReturningDateTime node, Object data) {
        return null;
    }

    public Object visit(ASTFunctionsReturningStrings node, Object data) {
        return null;
    }

    public Object visit(ASTTrimSpecification node, Object data) {
        return null;
    }

    public Object visit(ASTTrimCharacter node, Object data) {
        return null;
    }

    public Object visit(ASTPatternValue node, Object data) {
        return null;
    }

    public Object visit(ASTEscapeCharacter node, Object data) {
        return null;
    }

    public Object visit(ASTAbstractSchemaName node, Object data) {
        return null;
    }

    public Object visit(ASTIdentificationVariable node, Object data) {
        return null;
    }

    public Object visit(ASTLiteral node, Object data) {
        return null;
    }

    public Object visit(ASTStringLiteral node, Object data) {
        return null;
    }

    public Object visit(ASTNumericLiteral node, Object data) {
        return null;
    }

    public Object visit(ASTIntegerLiteral node, Object data) {
        return null;
    }

    public Object visit(ASTFloatingPointLiteral node, Object data) {
        return null;
    }

    public Object visit(ASTBooleanLiteral node, Object data) {
        return null;
    }

    public Object visit(ASTInputParameter node, Object data) {
        return null;
    }

    public Object visit(ASTPositionalParameter node, Object data) {
        return null;
    }

    public Object visit(ASTNamedParameter node, Object data) {
        return null;
    }

 }
