/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.workingset.filter;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

import org.objectweb.perseus.persistence.api.VirtualState;
import org.objectweb.perseus.persistence.api.WorkingSet;
import org.objectweb.perseus.persistence.api.WorkingSetFilter;
import org.objectweb.speedo.genclass.api.SpeedoGenClassPO;
import org.objectweb.speedo.mim.api.StateItf;

/**
 * This filter enables to order objects involved in the working set as follows:
 * -------------------------------------
 * | PersistentObjectItf | SpeedoGenClassPO |
 * -------------------------------------
 * @author Y.Bersihand
 */
public class WorkingSetGenClassFilter implements WorkingSetFilter {
	
	//remove all the GenClass from the ws map and put it in a tmp map
	//add the tmp map in the end of the ws map
	//the ws map is then ordered as described above
	public void order(WorkingSet ws) {
		//get the map (oid,entry)
		Map wsMap = ws.getOid2Entry();
		//get a copy to iterate over
		Map wsMapCopy = new LinkedHashMap(wsMap);
		//tmp map
		Map genClassMap = new LinkedHashMap();
		Iterator it = wsMapCopy.keySet().iterator();
		while (it.hasNext()) {
			Object oid = it.next();
            org.objectweb.perseus.persistence.api.State state = (org.objectweb.perseus.persistence.api.State) wsMap.get(oid);
			if (!(state instanceof VirtualState) && ((StateItf) state).getSpeedoPO() instanceof SpeedoGenClassPO) {
				//remove the element from the ws map
				//and add it in the tmp map
				genClassMap.put(oid, wsMap.remove(oid));
			}
		}
		//the wsMap contains only PersistentObjectItf
		//when can add all the SpeedoGenClassPO in the end of the map
		wsMap.putAll(genClassMap);
	}
}
