/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.ejb.pobjects.basic;

import junit.framework.Assert;

import java.io.Serializable;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.SecondaryTable;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

/**
 * This class defines some unique constraints.
 * @author P. Dechamboux
 */
@Entity
@Table(name="EJB_CONSTRAINTSDEFM",
		uniqueConstraints={
		@UniqueConstraint(columnNames={"f1","f2"})
		}
)
@SecondaryTable(name="EJB_CONSTRAINTSDEFS",
		uniqueConstraints={
		@UniqueConstraint(columnNames={"f4"}),
		@UniqueConstraint(columnNames={"f5","f3"}),
		@UniqueConstraint(columnNames={"f3","f5"})
		}
)
public class ConstraintsDef implements Serializable {
	@Id
	private Long oid;
	@Column(name="INTFIELD",unique=true)
	private int int_field;
	@Basic
	private int f1;
	@Basic
	private int f2;
	@Column(table="EJB_CONSTRAINTSDEFS")
	private int f3;
	@Column(table="EJB_CONSTRAINTSDEFS")
	private int f4;
	@Column(table="EJB_CONSTRAINTSDEFS")
	private int f5;
	
	public void assign(long longval) {
		this.int_field = (int) longval;
		this.f1 = this.int_field + 1;
		this.f2 = this.int_field + 2;
		this.f3 = this.int_field + 3;
		this.f4 = this.int_field + 4;
		this.f5 = this.int_field + 5;
	}

	public void check(long longval) {
		Assert.assertEquals("bad int_field value", int_field, (int) longval);
		Assert.assertEquals("bad f1 value", int_field, (int) longval + 1);
		Assert.assertEquals("bad f2 value", int_field, (int) longval + 2);
		Assert.assertEquals("bad f3 value", int_field, (int) longval + 3);
		Assert.assertEquals("bad f4 value", int_field, (int) longval + 4);
		Assert.assertEquals("bad f5 value", int_field, (int) longval + 5);
	}

	public int retrieve_int_field() {
		return int_field;
	}

	public void assign_int_field(int int_field) {
		this.int_field = int_field;
		this.f1 = this.int_field + 1;
		this.f2 = this.int_field + 2;
		this.f3 = this.int_field + 3;
		this.f4 = this.int_field + 4;
		this.f5 = this.int_field + 5;
	}
}
