/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.query;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.Alea;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestParallelQueries extends SpeedoTestHelper {

    private static TestQueries test;
    static {
        test = new TestQueries("TestParallelQueries");
    }

	private final static String[] METHOD_NAMES = {
		"test0Parameter",
		"test10rder",
		"test20rder",
		"test1Parameter",
		"testFieldParameter",
		"test2Parameters",
		"test3Parameters",
		"testMapParameters",
		"testArrayParameters",
		"testFieldRef",
		"testThis",
		"testParamRef",
		"testBsContainsParamA0B0",
		"testBsContainsParamA0B2",
		"testBsContainsVarB3EmptyShortPath",
		"testBsContainsVarB0ShortPath",
		"testBsContainsVarB1ShortPath",
		"testBsContainsVarB3EmptyLongPath",
		"testBsContainsVarB0LongPath",
		"testBsContainsVarB1LongPath",
		"testBsIsEmptyA0",
		"testBsIsEmptyA3",
		"testBsIsEmptyA0WithNot",
		"testBsIsEmptyA3WithNot",
		"testBsContainsVarB3EmptyLongPath",
		"testBsContainsVarB3EmptyLongPath",
		"testBsIsNotEmptyA0Global",
		"testBsIsNotEmptyA0Local",
		"testBsIsNotEmptyA3Global",
		"testNotA0",
		"testExtentAMMBfalse",
		"testExtentBMMBfalse",
		"testQueryBasedOnExtent",
		"testSequenceIdNavigateToPrimitive",
		"testSequenceIdNavigateToPrimitive2",
		"testMatches"
		//"testBsContains2Param_0_3"
	};

	public TestParallelQueries(String name) {
		super(name);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.query.TestParallelQueries";
	}

    private void launchTest(int queryidx, int threadId, int cpt) {
		String msg = "Thread " + threadId + "(" + cpt + "), method: "
		    + METHOD_NAMES[queryidx];
	    logger.log(BasicLevel.DEBUG, msg);
	    try {
			switch(queryidx) {
			case 0:
				test.test0Parameter();
				break;
			case 1:
				test.test10rder();
				break;
			case 2:
				test.test20rder();
				break;
			case 3:
				test.test1Parameter();
				break;
			case 4:
				test.testFieldParameter();
				break;
			case 5:
				test.test2Parameters();
				break;
			case 6:
				test.test3Parameters();
				break;
			case 7:
				test.testMapParameters();
				break;
			case 8:
				test.testArrayParameters();
				break;
			case 9:
				test.testFieldRef();
				break;
			case 10:
				test.testThis();
				break;
			case 11:
				test.testParamRef();
				break;
			case 12:
				test.testBsContainsParamA0B0();
				break;
			case 13:
				test.testBsContainsParamA0B2();
				break;
			case 14:
				test.testBsContainsVarB3EmptyShortPath();
				break;
			case 15:
				test.testBsContainsVarB0ShortPath();
				break;
			case 16:
				test.testBsContainsVarB1ShortPath();
				break;
			case 17:
				test.testBsContainsVarB3EmptyLongPath();
				break;
			case 18:
				test.testBsContainsVarB0LongPath();
				break;
			case 19:
				test.testBsContainsVarB1LongPath();
				break;
			case 20:
				test.testBsIsEmptyA0();
				break;
			case 21:
				test.testBsIsEmptyA3();
				break;
			case 22:
				test.testBsIsEmptyA0WithNot();
				break;
			case 23:
				test.testBsIsEmptyA3WithNot();
				break;
			case 24:
				test.testBsContainsVarB3EmptyLongPath();
				break;
			case 25:
				test.testBsContainsVarB3EmptyLongPath();
				break;
			case 26:
				test.testBsIsNotEmptyA0Global();
				break;
			case 27:
				test.testBsIsNotEmptyA0Local();
				break;
			case 28:
				test.testBsIsNotEmptyA3Global();
				break;
			case 29:
				test.testNotA0();
				break;
			case 30:
				test.testExtentAMMBfalse();
				break;
			case 31:
				test.testExtentBMMBfalse();
				break;
			case 32:
				test.testQueryBasedOnExtent();
				break;
			case 33:
				test.testSequenceIdNavigateToPrimitive();
				break;
			case 34:
				test.testSequenceIdNavigateToPrimitive2();
				break;
			case 35:
				test.testMatches();
				break;
			case 36:
				test.testBsContains2Param_0_3();
				break;
			}
        } catch (Exception e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, msg, ie);
            fail(msg + ie.getMessage());
        }
    }

	private static long queryExecTime = 0;
	private static long nbquery = 0;
	private static long errors = 0;
	private static long nbprint = 1000;
	private synchronized static void queryError(Logger logger, Throwable t) {
		logger.log(BasicLevel.ERROR, "", t);
		errors ++;
	}
	private synchronized static void queryFinished(long time, Logger logger) {
		nbquery++;
		queryExecTime += time;
		if ((nbquery % nbprint) == 0) {
			logger.log(BasicLevel.INFO, ".");
		}
	}

    public void test1() {
        final int nbThread = getIntProperty(getLoggerName() +".nbthread", 15);
        logger.log(BasicLevel.DEBUG, getLoggerName() + ".nbthread=" + nbThread);
        final int nbQuery = getIntProperty(getLoggerName() +".nbquery", 500);
        logger.log(BasicLevel.DEBUG, getLoggerName() + ".nbquery=" + nbQuery);
        nbprint = getIntProperty(getLoggerName() +".nbprint", nbQuery);
		ArrayList al = new ArrayList(50);
		String v = System.getProperty(getLoggerName() +".query");
		if (v != null) {
			v = v.trim();
			if (!v.startsWith("$")) {
				StringTokenizer st = new StringTokenizer(v, ", ", false);
				while(st.hasMoreTokens()) {
					try {
						al.add(new Integer(st.nextToken()));
					} catch (NumberFormatException e) {
					}
				}
			}
		}
		if (al.size() == 0) {
			for(int i=0;i<METHOD_NAMES.length; i++) {
				al.add(new Integer(i));
			}
		}
		final int[] queries = new int[al.size()];
		for(int i=(al.size()-1); i>=0; i--) {
			queries[i] = ((Integer) al.get(i)).intValue();
		}
		logger.log(BasicLevel.INFO, "Test: nbthread=" + nbThread
				+ ", nbquery=" + nbQuery
				+ ", queryIdx=" + al);
        Thread[] ts = new Thread[nbThread];
        for(int i=0; i<nbThread; i++) {
            final int threadId = i;
            ts[i] = new Thread(
                    new Runnable () {
                        public void run () {
	                        if (nbQuery == 0) {
		                        while(true) {
			                        try {
				                        launchTest(queries[Alea.rand(0, queries.length-1)], threadId, -1);
				                        queryFinished(0, logger);
			                        } catch (Throwable e) {
				                        logger.log(BasicLevel.ERROR, "Thread " + threadId + " fail!", e);
				                        break;
			                        }
		                        }
	                        } else {
		                        for(int j=0; j<nbQuery; j++) {
			                        long exectime = System.currentTimeMillis();
			                        try {
				                        launchTest(queries[Alea.rand(0, queries.length-1)], threadId, j);
				                        queryFinished(System.currentTimeMillis() - exectime, logger);
			                        } catch (Throwable e) {
				                        queryError(logger, e);
			                        }
		                        }
	                        }
                        }
                    }
            );
        }
	    long exectime = System.currentTimeMillis();
        for(int i=0; i<nbThread; i++) {
            ts[i].start();
        }
        try {
            for(int i=0; i<nbThread; i++) {
                ts[i].join();
            }
        } catch (InterruptedException e) {
            fail(e.getMessage());
        }
	    if (nbQuery > 0) {
			exectime = System.currentTimeMillis() - exectime;
			int nbRunQuery =  (nbThread * nbQuery);
			logger.log(BasicLevel.INFO, "Query successed: "
					+ ((nbquery * 100) / nbRunQuery) + "% ("
					+ nbquery + "/" + nbRunQuery + ", "
					+ (nbRunQuery - nbquery) + " error)");
			logger.log(BasicLevel.INFO, "Query average execution time: "
					+ (queryExecTime / nbquery) +"ms");
			logger.log(BasicLevel.INFO, "Rate: " + ((nbquery * 1000) / exectime) + " query/sec");
			if (errors > 0) {
				fail(errors + " errors occured!");
			}
	    }
    }
}
