/*
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Created on 15 mars 2004
 * @author fmillevi@yahoo.com
 *
 */
package org.objectweb.speedo.j2eedo.bo;

import java.util.Calendar;
import java.util.Iterator;

import javax.jdo.JDOException;
import javax.jdo.PersistenceManager;

import org.objectweb.speedo.Alea;
import org.objectweb.speedo.j2eedo.database.Address;
import org.objectweb.speedo.j2eedo.database.Department;
import org.objectweb.speedo.j2eedo.database.Employee;
import org.objectweb.speedo.j2eedo.database.Project;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

/**
 * This class define the Java class to methods used to handle department
 * @author fmillevi@yahoo.com
 */
public class DepartmentFactory {
    static Logger logger = Monolog.initialize().getLogger(DatabaseImpl.class.getName());
	/**
	 * the constant <code>MIN_EMPLOYEE_PER_DEPARTMENT</code> defines
	 * the minimum employee number to be created per department
	 */
	public static final int MIN_EMPLOYEE_PER_DEPARTMENT = 10;
	/**
	 * the constant <code>MAX_EMPLOYEE_PER_DEPARTMENT</code> defines
	 * the maximum employee number to be created per department
	 */
	public static final int MAX_EMPLOYEE_PER_DEPARTMENT = 70;

	/**
	 * Create a new department and it's employees
	 * @throws JDOException if error occurs while creating the deparment
	 * @throws Exception
	 */
	public void newDepartmentWithEmployees(
            PollsSynchronizations pollsSync, 
            StringBuffer outStr, 
            PersistenceManager pm) {
        outStr.append("\nCreate new deparment with its employees");
		Department d = new Department("Dept-" + Alea.randomstring(4, 9));
		pm.makePersistent(d);
        pollsSync.addInPool(DatabaseImpl.poolOfDepartmentId, d.getDeptid());
		logger.log(BasicLevel.DEBUG,
				"Create a new department id :" + d.getDeptid());
		int nbEmployees = Alea.rand(MIN_EMPLOYEE_PER_DEPARTMENT, MAX_EMPLOYEE_PER_DEPARTMENT);
        outStr.append("\n\tAdd new Department : ").append(d.getDeptid());
        outStr.append(" with ").append(nbEmployees).append(" employees");
        Employee boss = null;
		for (int i = 0; i < nbEmployees; i++) {
			Calendar cal = Calendar.getInstance();
			cal.set(Alea.rand(1900, 2004), Alea.rand(1, 12), Alea.rand(1, 28));
			Employee e = new Employee(
						"First-" + Alea.randomstring(4, 5),
						"Last-" + Alea.randomstring(8, 10),
						cal.getTime(),
						d);
			e.setSalary(Alea.rand(5000, 45000));
			Address a = new Address();
			a.setCity("City-" + Alea.randomstring(4, 5));
			a.setStreet("Street-" + Alea.randomstring(4, 5));
			a.setState("StateItf-" + Alea.randomstring(4, 5));
			a.setZipcode("ZIP-" + Alea.randomstring(2, 3));
			e.setAddress(a);
			pm.makePersistent(e);
			if (0 == i) {
				boss = e;
                outStr.append("\nThe boss: ");
			} else {
				e.setManager(boss);
			}
            outStr.append(e.getAsString());
			pollsSync.addInPool(DatabaseImpl.poolOfEmployeeId, e.getEmpid());
            
		}
		pm.setUserObject(pollsSync);
	}

	/**
	 * Gets the department definition
	 * @throws JDOException if error occurs while getting the deparment
	 * @throws Exception
	 */
	public void getDepartment(StringBuffer outStr, PersistenceManager pm) {
		long id = DatabaseImpl.getDepartmentIdFromPool();
		logger.log(BasicLevel.DEBUG, "Get Department having id :" + id);
		outStr.append("\nGet the department id :").append(id);
		Department d = (Department) pm.getObjectById(
					pm.newObjectIdInstance(Department.class, Long.toString(id)),
					false);
		outStr.append(d.getAsString());
	}

	/**
	 * Affects all employees and project to an other department and removes the former department
	 * @throws JDOException
	 * @throws Exception
	 */
	public void mergeDepartment(PollsSynchronizations pollsSync, 
            StringBuffer outStr, PersistenceManager pm) {
		long initialDeptid = DatabaseImpl.getDepartmentIdFromPool();
		long destinationDeptid = DatabaseImpl.getDepartmentIdFromPool();
		while (initialDeptid == destinationDeptid) {
			destinationDeptid = DatabaseImpl.getDepartmentIdFromPool();
        }

		// remove id from static pool to avoid other threatment to use it
		pollsSync.removeFromPool(DatabaseImpl.poolOfDepartmentId, initialDeptid);
		pm.setUserObject(pollsSync);

		outStr.append("\nRemove Dept : " + initialDeptid);
		logger.log(BasicLevel.DEBUG, "Delete the department id :" + initialDeptid);

		Department initialDepartment =
			(Department) pm.getObjectById(
					pm.newObjectIdInstance(
							Department.class,
							Long.toString(initialDeptid)),
							false);

		outStr.append("\nMerge with Dept : " + destinationDeptid);
		Department destinationDepartment =
			(Department) pm.getObjectById(
					pm.newObjectIdInstance(
							Department.class,
							Long.toString(destinationDeptid)),
							false);

		// move projects
		Iterator projects = initialDepartment.getProjects().iterator();
		Project p = null;
		while (projects.hasNext()) {
			p = (Project) projects.next();
			p.setDepartment(destinationDepartment);
			outStr.append("\nMove project : ").append(p.getId());
		}

		// move employees
		Iterator persons = initialDepartment.getEmployees().iterator();
		Employee e = null;
		while (persons.hasNext()) {
			e = (Employee) persons.next();
			e.setDepartment(destinationDepartment);
			outStr.append("\nAffect employee : ").append(e.getId());
		}

		// remove department
		pm.deletePersistent(initialDepartment);
	}

	/**
	 * Creates a new department, selects an existing department and shifts to the
	 * new department :
	 * <ul><li>50% of the projects</li><li>50% of the employees</li></ul>
	 * @throws JDOException
	 * @throws Exception
	 */
	public void splitDepartment(PollsSynchronizations pollsSync, 
            StringBuffer outStr, 
            PersistenceManager pm) {
		long initialDeptid = DatabaseImpl.getDepartmentIdFromPool();
		outStr.append("\nSplit Dept : " + initialDeptid);
		logger.log(BasicLevel.DEBUG,
				"Split the department id :" + initialDeptid);

		Department initialDepartment =
			(Department) pm.getObjectById(
					pm.newObjectIdInstance(
							Department.class,
							Long.toString(initialDeptid)),
							false);

		Department d = new Department(initialDepartment.getName()+"-splited");
		pm.makePersistent(d);

		pollsSync.addInPool(DatabaseImpl.poolOfDepartmentId, d.getDeptid());
		pm.setUserObject(pollsSync);

		outStr.append("\n with Dept : " + d.getDeptid() );

		// move projects
		Iterator projects = initialDepartment.getProjects().iterator();
		Project p = null;
		int nbProject = Math.round(initialDepartment.getProjects().size()/2);
		while (projects.hasNext() && nbProject>0) {
			nbProject--;
			p = (Project) projects.next();
			p.setDepartment(d);
			outStr.append("\nMove project : " + p.getId());
		}

		// move employees
		Iterator persons = initialDepartment.getEmployees().iterator();
		Employee e = null;
		int nbEmployee = Math.round(initialDepartment.getEmployees().size()/2);
		while (persons.hasNext() && nbEmployee >0) {
			nbEmployee--;
			e = (Employee) persons.next();
			e.setDepartment(d);
			outStr.append("\nAffect employee : " + e.getId());
		}
	}

	/**
	 * Selects a department, gets the list of employees. The first returned employee is the
	 * new manager for the following employees
	 * @throws JDOException
	 * @throws Exception
	 */
	public void setManagerForADepartment(
            StringBuffer outStr, 
            PersistenceManager pm) {
        outStr.append("\nUpdate the boss for a department's employees");
		Department d = (Department) pm.getObjectById(
					pm.newObjectIdInstance(Department.class,
							Long.toString(DatabaseImpl.getDepartmentIdFromPool())),
							false);
		logger.log(BasicLevel.DEBUG,
				"Update manager for the department having id :" + d.getDeptid());
		// the first person from a department is the boss;
		Iterator persons = d.getEmployees().iterator();
		Employee boss = null;
		Employee otherEmp = null;
		if (persons.hasNext()) {
			boss = (Employee) persons.next();
			outStr.append("\nThe boss of the department '");
			outStr.append(boss.getDepartment().getName());
			outStr.append("' is now: ").append(boss.getFirstname());
			while (persons.hasNext()) {
				otherEmp = (Employee) persons.next();
				otherEmp.setManager(boss);
                outStr.append("\nThe employee ").append(otherEmp.getEmpid());
                outStr.append(" has a new manager.");
			}
		}
	}
}
