/*
 * Speedo: an implementation of JDO compliant personality on top of JORM
 * generic I/O sub-system. Copyright (C) 2001-2004 France Telecom R&D
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Release: 1.0
 * 
 * Created on 27 feb. 2004 @author fmillevi@yahoo.com
 *  
 */
package org.objectweb.speedo.j2eedo.database;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * This class define the Java class to access to the department's data
 * @author fmillevi@yahoo.com
 *  
 */
public class Department implements DatabaseObjectInterface {
	private long deptid;
	private String name;
	private Collection employees; //element:Employee
	private Collection projects; //element:Project

	/**
	 * Empty constructor
	 */
	public Department() {
		super();
	}

	public String getAsString() {
		StringBuffer sb = new StringBuffer();
		sb.append("\nDepartment (deptid:").append(this.deptid);
		sb.append(")\n\tName:").append(this.name);
		if (null != this.employees) {
			sb.append("\n\tEmployees:");
			Iterator employeeIterator = employees.iterator();
			Employee e = null;
			while (employeeIterator.hasNext()) {
				e = (Employee) employeeIterator.next();
				sb.append("\n\t\t").append(e.getFirstname()).append(" ");
				sb.append(e.getLastname());
			}
		}
		if (projects != null) {
			Iterator projectIterator = projects.iterator();
			if (projectIterator.hasNext()) {
				sb.append("\n\tProjects:");
			}
			while (projectIterator.hasNext()) {
				sb.append("\n\t\t");
				sb.append(((Project) projectIterator.next()).getName());
			}
		}
		return sb.toString();
	}
	/**
	 * @see org.objectweb.speedo.j2eedo.database.DatabaseObjectInterface#getId()
	 */
	public long getId() {
		return this.deptid;
	}

	public Department(String itsName) {
		this.name = itsName;
		employees = new ArrayList();
		projects = new ArrayList();
	}
	/**
	 * @return Returns the deptid.
	 */
	public long getDeptid() {
		return deptid;
	}

	/**
	 * @return Returns the employees.
	 */
	public Collection getEmployees() {
		return employees;
	}

	/**
	 * @param employees
	 *            The employees to set.
	 */
	public void setEmployees(Collection employees) {
		this.employees = employees;
	}

	/**
	 * @param newEmployee
	 */
	public void addEmployee(Employee newEmployee) {
		employees.add(newEmployee);
	}

	/**
	 * @param newEmployee
	 */
	public void removeEmployee(Employee newEmployee) {
		employees.remove(newEmployee);
	}

	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return Returns the projects.
	 */
	public Collection getProjects() {
		return projects;
	}

	/**
	 * @param projects
	 *            The projects to set.
	 */
	public void setProjects(Collection projects) {
		this.projects.addAll(projects);
	}

	/**
	 * @param newProject
	 */
	public void addProject(Project newProject) {
		projects.add(newProject);
	}
	/**
	 * @param newProject
	 */
	public void removeProject(Project newProject) {
		projects.remove(newProject);
	}
}